#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"IntArray.h"


// Notes.


/*
	Optimization will replace multiplications and divisions
	with bitwise operators.
*/


//
// Bit array.
//


// Macros.


// Number of bits in element.
#define		EL_NBIT		( sizeof( BAryEl ) * 8 )

// Number of bits and elements in global block.
#define		RGL_NBIT	( 1 << 8 )		// 256.
#define		RGL_NEL		( RGL_NBIT / EL_NBIT )

// Number of ones in global block.
#define		SLO_MAX		( 1 << ( sizeof( BArySel ) * 8 ) )	// 65,536.
#define		SGL_NONE	( 1 << 11 )		// 2,048.
#define		SLO_NONE	( 1 << 6 )		// 64 (64x32=2,048).
#define		SGL_NEL		( SGL_NONE / SLO_NONE )		// 128.


// Types.


// Integers.
typedef		unsigned long	BAryBit;	// Bit.
typedef		unsigned long	BAryIdx;	// Index.
typedef		unsigned long	BAryEl;		// Element of array.
typedef		unsigned char	BAryRank;	// Local index of rank.
typedef		unsigned short	BArySel;	// Local index of select.

// Bit array.
typedef		struct BAry
{
	BAryIdx		nbit;	// Number of bits.
	BAryIdx		nel;	// Number of elements.
	BAryEl		*el;	// Bit array.

	// If index are built, following members will be available.
	BAryIdx		nzero;		// Number of 0-bits.
	BAryIdx		none;		// Number of 1-bits.

	// Index for rank.
	BAryIdx		nrank_g;	// Number of global blocks.
	BAryIdx		nrank_l;	// Number of local blocks.
	BAryIdx		*rank_g;	// Rank for global.
	BAryRank	*rank_l;	// Rank for local.

	// Index for select.
	BAryIdx		nsel_g;		// Number of global blocks.
	BAryIdx		nsel_l;		// Number of local blocks.
	BAryIdx		*sel_g;		// Select for global.
	BArySel		*sel_l;		// Select for local.
}	BAry, *hBAry;


// Initialize and finish bit array.


// Get the number of elements.
static	BAryIdx		BAry_ElNum( BAryIdx nbit )
{
	return	( nbit + EL_NBIT - 1 ) / EL_NBIT;
}

// Get the number of global blocks for rank.
static	BAryIdx		BAry_RankGlobalNum( BAryIdx nbit )
{
	return	( nbit + RGL_NBIT - 1 ) / RGL_NBIT;
}

// Get the number of local blocks for rank.
static	BAryIdx		BAry_RankLocalNum( BAryIdx nbit )
{
	return	( nbit + EL_NBIT - 1 ) / EL_NBIT;
}

// Get the number of global blocks for select.
static	BAryIdx		BAry_SelectGlobalNum( BAryIdx none )
{
	return	( none + SGL_NONE - 1 ) / SGL_NONE;
}

// Get the number of local blocks for select.
static	BAryIdx		BAry_SelectLocalNum( BAryIdx none )
{
	return	( none + SLO_NONE - 1 ) / SLO_NONE;
}

// Initialize index for rank.
static	void	BAry_InitRank( hBAry h )
{
	h->rank_g = NULL;
	h->rank_l = NULL;
}

// Initialize index for select.
static	void	BAry_InitSelect( hBAry h )
{
	h->sel_g = NULL;
	h->sel_l = NULL;
}

// Initialize index.
static	void	BAry_InitIndex( hBAry h )
{
	BAry_InitRank( h );
	BAry_InitSelect( h );
}

// Initialize.
static	void	BAry_Init( hBAry h )
{
	memset( h, 0, sizeof( *h ) );

	h->el = NULL;
	BAry_InitIndex( h );
}

// Create.
static	hBAry	BAry_Create( void )
{
	hBAry	h;

	if ( ( h = ( hBAry )malloc( sizeof( *h ) ) ) != NULL )
		BAry_Init( h );

	return	h;
}

// Finish index for rank.
static	void	BAry_FinishRank( hBAry h )
{
	if ( h->rank_g != NULL )
		free( h->rank_g );
	if ( h->rank_l != NULL )
		free( h->rank_l );

	BAry_InitRank( h );
}

// Finish index for select.
static	void	BAry_FinishSelect( hBAry h )
{
	if ( h->sel_g != NULL )
		free( h->sel_g );
	if ( h->sel_l != NULL )
		free( h->sel_l );

	BAry_InitSelect( h );
}

// Finish index.
static	void	BAry_FinishIndex( hBAry h )
{
	BAry_FinishRank( h );
	BAry_FinishSelect( h );
}

// Finish.
static	void	BAry_Finish( hBAry h )
{
	if ( h->el != NULL )
		free( h->el );

	BAry_FinishIndex( h );
}

// Destroy.
static	void	BAry_Destroy( hBAry h )
{
	BAry_Finish( h );
	free( h );
}


// Population count.


// Population count (from Hacker's Delight).
static	BAryIdx		BAry_Pop( BAryEl x )
{
	x -= ( x >> 1 ) & 0x55555555;
	x = ( x & 0x33333333 ) + ( ( x >> 2 ) & 0x33333333 );
	x = ( x + ( x >> 4 ) ) & 0x0F0F0F0F;
	x += x >> 8;
	x += x >> 16;

	return	x & 0x3F;
}

// Population count keeping counts in progress (from Hacker's Delight).
static	BAryIdx		BAry_PopX( BAryEl x )
{
	x -= ( x >> 1 ) & 0x55555555;
	x = ( x & 0x33333333 ) + ( ( x >> 2 ) & 0x33333333 );
	x = ( x + ( x >> 4 ) ) & 0x0F0F0F0F;
	x += ( x & 0x0F000F ) << 8;
	x += ( x & 0xFF00 ) << 16;

	return	x;
}


// Index.


// Allocate memory to index for rank.
static	int		BAry_AllocRank( hBAry h )
{
	BAryIdx		size;

	BAry_FinishRank( h );

	h->nrank_g = BAry_RankGlobalNum( h->nbit );
	size = sizeof( BAryIdx ) * ( h->nrank_g + 1 );
	if ( ( h->rank_g = ( BAryIdx * )malloc( size ) ) == NULL )
		return	IARY_EMEM;

	h->nrank_l = BAry_RankLocalNum( h->nbit );
	size = sizeof( BAryRank ) * ( h->nrank_l + 1 );
	if ( ( h->rank_l = ( BAryRank * )malloc( size ) ) == NULL )
		return	IARY_EMEM;

	return	IARY_OK;
}

// Allocate memory to index for select.
static	int		BAry_AllocSelect( hBAry h )
{
	BAryIdx		i, size, none = 0;

	BAry_FinishSelect( h );

	for ( i = 0; i < h->nel; i++ )
		none += BAry_Pop( h->el[i] );
	h->none = none;
	h->nzero = h->nbit - none;

	h->nsel_g = BAry_SelectGlobalNum( h->none );
	size = sizeof( BAryIdx ) * ( h->nsel_g + 1 );
	if ( ( h->sel_g = ( BAryIdx * )malloc( size ) ) == NULL )
		return	IARY_EMEM;

	h->nsel_l = BAry_SelectLocalNum( h->none );
	size = sizeof( BArySel ) * ( h->nsel_l + 1 );
	if ( ( h->sel_l = ( BArySel * )malloc( size ) ) == NULL )
		return	IARY_EMEM;

	return	IARY_OK;
}


// Interfaces.


// Resize bit array.
static	int		BAry_Resize( hBAry h, BAryIdx nbit )
{
	BAryEl	*el;
	BAryIdx		nel;

	if ( ( nel = BAry_ElNum( nbit ) ) != h->nel )
	{
		if ( ( el = realloc( h->el, sizeof( *el ) * nel ) ) == NULL )
			return	IARY_EMEM;
		h->el = el;

		if ( nel > h->nel )
			memset( el + h->nel, 0, sizeof( *el ) * ( nel - h->nel ) );
	}

	h->nbit = nbit;
	h->nel = nel;

	return	IARY_OK;
}

// Create bit array, and clear the array by zero.
static	int		BAry_Open( hBAry *h, BAryIdx nbit )
{
	int		ret;

	if ( ( *h = BAry_Create() ) == NULL )
		return	IARY_EMEM;

	if ( ( ret = BAry_Resize( *h, nbit ) ) != IARY_OK )
	{
		BAry_Destroy( *h );
		*h = NULL;
	}

	return	ret;
}

// Destroy bit array.
static	void	BAry_Close( hBAry h )
{
	BAry_Destroy( h );
}

// Load bit array without index from file.
static	int		BAry_Load( hBAry *h, FILE *fp )
{
	int		ret;
	hBAry	ary;
	BAryIdx		nbit;

	*h = NULL;
	if ( fread( &nbit, sizeof( nbit ), 1, fp ) != 1 )
		return	IARY_EIO;

	if ( ( ret = BAry_Open( &ary, nbit ) ) != IARY_OK )
		return	ret;

	if ( fread( ary->el, sizeof( *ary->el ), ary->nel, fp ) != ary->nel )
	{
		BAry_Close( ary );
		return	IARY_EIO;
	}
	*h = ary;

	return	IARY_OK;
}

// Save bit array without inde to filex.
static	int		BAry_Save( hBAry h, FILE *fp )
{
	if ( fwrite( &h->nbit, sizeof( h->nbit ), 1, fp ) != 1 ||
		 fwrite( h->el, sizeof( *h->el ), h->nel, fp ) != h->nel )
		return	IARY_EIO;

	return	IARY_OK;
}

// Set bit.
void	BAry_SetBit( hBAry h, BAryIdx idx, BAryBit bit )
{
	if ( bit )
		h->el[ idx / EL_NBIT ] |= ( BAryEl )1 << ( idx % EL_NBIT );
	else
		h->el[ idx / EL_NBIT ] &= ~( ( BAryEl )1 << ( idx % EL_NBIT ) );
}

// Get bit.
static	BAryBit		BAry_GetBit( hBAry h, BAryIdx idx )
{
	return	( h->el[ idx / EL_NBIT ] >> ( idx % EL_NBIT ) ) & 1;
}

// Build index for rank.
static	int		BAry_BuildRank( hBAry h )
{
	int		ret;
	BAryIdx		i, pop, sum = 0;

	if ( ( ret = BAry_AllocRank( h ) ) != IARY_OK )
		return	ret;

	for ( i = 0; i < h->nel; )
	{
		h->rank_g[ i / RGL_NEL ] = sum;
		h->rank_l[i] = 0;
		do
		{
			pop = BAry_Pop( h->el[i] );
			h->rank_l[ i + 1 ] = h->rank_l[i] + ( BAryRank )pop;
			sum += pop;
		}	while ( ( ++i & ( RGL_NEL - 1 ) ) && i < h->nel );
	}
	h->rank_g[ h->nrank_g ] = sum;
	h->none = sum;
	h->nzero = h->nbit - h->none;

	return	IARY_OK;
}

// Build index for select.
static	int		BAry_BuildSelect( hBAry h )
{
	int		ret;
	BAryEl	x;
	BAryIdx		i, j, idx, ct = 0, old = 0;

	if ( ( ret = BAry_AllocSelect( h ) ) != IARY_OK )
		return	ret;

	for ( i = 0; i < h->nel; i++ )
	{
		for ( x = h->el[i], j = 0; x; x >>= 1, j++ )
		{
			if ( x & 1 )
			{
				idx = i * EL_NBIT + j;
				if ( !( ct % SGL_NONE ) )
					h->sel_g[ ct / SGL_NONE ] = old = idx;
				if ( !( ct % SLO_NONE ) )
					h->sel_l[ ct / SLO_NONE ] = ( BArySel )( idx - old );
				ct++;
			}
		}
	}

	return	IARY_OK;
}

// Rank.
static	BAryIdx		BAry_Rank( hBAry h, BAryIdx idx )
{
	BAryIdx		idx_g, idx_l, rank;

	idx_g = idx / RGL_NBIT;
	idx_l = idx / EL_NBIT;

	rank = h->rank_g[idx_g] + h->rank_l[idx_l];
	rank += BAry_Pop( h->el[idx_l] &
		( ( ( BAryIdx )1 << ( idx % EL_NBIT ) ) - 1 ) );

	return	rank;
}

// Select.
static	BAryIdx		BAry_Select( hBAry h, BAryIdx n )
{
	BAryEl	el;
	BAryIdx		idx, pop;

	if ( n >= h->none )
		return	h->nbit;

	idx = h->sel_g[ n / SGL_NONE ] + h->sel_l[ n / SLO_NONE ];
	n = ( n % SLO_NONE ) + 1;
	el = h->el[ idx / EL_NBIT ] >> ( idx % EL_NBIT );

	pop = BAry_PopX( el );
	if ( ( pop >> 24 ) < n )
	{
		idx /= EL_NBIT;
		do
		{
			n -= pop >> 24;
			el = h->el[ ++idx ];
			pop = BAry_PopX( el );
		}	while ( ( pop >> 24 ) < n );
		idx *= EL_NBIT;
	}

	// Under 16 bits.
	if ( ( ( pop & 0xFF00 ) >> 8 ) < n )
	{
		el >>= 16;
		idx += 16;
		n -= ( pop & 0xFF00 ) >> 8;
		pop >>= 16;
	}

	// Under 8 bits.
	pop &= 0xFF;
	if ( pop < n )
	{
		el >>= 8;
		idx += 8;
		n -= pop;
	}

	while ( n )
	{
		n -= el & 1;
		el >>= 1;
		idx++;
	}

	return	idx - 1;
}

// Get size.
static	BAryIdx		BAry_Size( hBAry h )
{
	BAryIdx		size;

	size = sizeof( *h );
	size += h->nel * sizeof( *h->el );
	if ( h->rank_g != NULL )
		size += h->nrank_g * sizeof( *h->rank_g );
	if ( h->rank_l != NULL )
		size += h->nrank_l * sizeof( *h->rank_l );
	if ( h->sel_g != NULL )
		size += h->nsel_g * sizeof( *h->sel_g );
	if ( h->sel_l != NULL )
		size += h->nsel_l * sizeof( *h->sel_l );

	return	size;
}


//
// Integer array.
//


// Types.


// Compressed integer array.
typedef		struct IAry
{
	IAryIdx		nint;	// Number of integers.
	IAryIdx		nbit;	// Number of bits for len and val.
	IAryIdx		szint;	// Size of flag.
	IAryIdx		szbit;	// Size of len and val.
	hBAry		flag;	// Non-zero flags.
	hBAry		len;	// Numbers of bits for integers.
	hBAry		val;	// Integers.
}	IAry;


// Initialize and finish.


// Initialize.
static	void	IAry_Init( hIAry h )
{
	memset( h, 0, sizeof( *h ) );
	h->flag = NULL;
	h->len = NULL;
	h->val = NULL;
}

// Create.
static	hIAry	IAry_Create( void )
{
	hIAry	h;

	if ( ( h = ( hIAry )malloc( sizeof( *h ) ) ) != NULL )
		IAry_Init( h );

	return	h;
}

// Finish.
static	void	IAry_Finish( hIAry h )
{
	if ( h->flag != NULL )
		BAry_Close( h->flag );
	if ( h->len != NULL )
		BAry_Close( h->len );
	if ( h->val != NULL )
		BAry_Close( h->val );
}

// Destroy.
static	void	IAry_Destroy( hIAry h )
{
	IAry_Finish( h );
	free( h );
}


// Resize.


// Resize flag.
static	int		IAry_ResizeFlag( hIAry h, IAryIdx nint )
{
	int		ret;

	if ( ( ret = BAry_Resize( h->flag, nint ) ) != IARY_OK )
		return	ret;
	h->szint = nint;

	return	IARY_OK;
}

// Resize len and val.
static	int		IAry_ResizeOther( hIAry h, IAryIdx nbit )
{
	int		ret;

	if ( ( ret = BAry_Resize( h->len, nbit + 1 ) ) != IARY_OK ||
		 ( ret = BAry_Resize( h->val, nbit ) ) != IARY_OK )
		return	ret;
	h->szbit = nbit;

	return	IARY_OK;
}

// Expand arrays.
static	int		IAry_Alloc( hIAry h, IAryLen len )
{
	int		ret;
	IAryIdx		size;

	if ( h->nint >= h->szint )
	{
		size = h->nint + ( h->nint >> 1 ) + 1;
		if ( ( ret = IAry_ResizeFlag( h, size ) ) != IARY_OK )
			return	ret;
	}

	size = h->nbit + len;
	if ( size >= h->szbit )
	{
		size += ( size >> 1 ) + 1;
		if ( ( ret = IAry_ResizeOther( h, size ) ) != IARY_OK )
			return	ret;
	}

	return	IARY_OK;
}

// Prepare arrays.
static	int		IAry_Empty( hIAry h )
{
	int		ret;

	if ( ( ret = BAry_Open( &h->flag, 0 ) ) != IARY_OK ||
		 ( ret = BAry_Open( &h->len, 1 ) ) != IARY_OK ||
		 ( ret = BAry_Open( &h->val, 0 ) ) != IARY_OK )
		return	ret;

	BAry_SetBit( h->len, 0, 1 );

	return	IARY_OK;
}


// Build.


// Get the number of bits for integer.
static	IAryLen		IAry_IntLen( IAryInt x )
{
	IAryLen		len = 0;
	IAryUInt	y;

	y = ( x < 0 ) ? -x : x;
	while ( y )
		y >>= 1, len++;

	return	len;
}

// Get the number of bits for unsigned integer.
static	IAryLen		IAry_UIntLen( IAryUInt x )
{
	IAryLen		len = 0;

	x++;
	while ( x >>= 1 )
		len++;

	return	len;
}

// Put integer.
static	void	IAry_PutInt( hIAry h, IAryInt x )
{
	IAryLen		len = 0;
	IAryIdx		pos;
	IAryUInt	y;

	BAry_SetBit( h->flag, h->nint++, x != 0 );
	if ( !x )
		return;

	pos = h->nbit;
	BAry_SetBit( h->len, pos, 1 );
	BAry_SetBit( h->val, pos++, x < 0 );

	y = ( x < 0 ) ? -x : x;
	while ( y >> len )
		len++;

	for ( len--; len--; pos++ )
		BAry_SetBit( h->val, pos, ( y >> len ) & 1 );
	BAry_SetBit( h->len, pos, 1 );

	h->nbit = pos;
}

// Put unsigned integer.
static	void	IAry_PutUInt( hIAry h, IAryUInt x )
{
	IAryLen		len = 0;
	IAryIdx		pos;

	BAry_SetBit( h->flag, h->nint++, x != 0 );
	if ( !x )
		return;

	pos = h->nbit;
	BAry_SetBit( h->len, pos, 1 );

	x++;
	while ( x >> len )
		len++;

	for ( len--; len--; pos++ )
		BAry_SetBit( h->val, pos, ( x >> len ) & 1 );
	BAry_SetBit( h->len, pos, 1 );

	h->nbit = pos;
}


// Get integer.


// Get integer.
static	IAryInt		IAry_IntPos( hIAry h, IAryIdx *pos )
{
	BAryBit		sign;
	IAryUInt	val = 1;

	sign = BAry_GetBit( h->val, *pos );
	while ( !BAry_GetBit( h->len, ++*pos ) )
		val = ( val << 1 ) | BAry_GetBit( h->val, *pos );

	return	sign ? -val : val;
}

// Get unsigned integer.
static	IAryUInt	IAry_UIntPos( hIAry h, IAryIdx *pos )
{
	IAryUInt	val = 1;

	do
		val = ( val << 1 ) | BAry_GetBit( h->val, *pos );
	while ( !BAry_GetBit( h->len, ++*pos ) );

	return	val - 1;
}

// Get posistion.
static	IAryIdx		IAry_GetPos( hIAry h, IAryIdx idx )
{
	IAryIdx		rank, pos;

	rank = BAry_Rank( h->flag, idx );
	pos = BAry_Select( h->len, rank );

	return	pos;
}

// Jump position.
static	void	IAry_JumpPos( hIAryScan scan )
{
	if ( scan->jump && BAry_GetBit( scan->h->flag, scan->idx ) )
	{
		scan->pos = IAry_GetPos( scan->h, scan->idx );
		scan->jump = 0;
	}
	scan->idx++;
}


// Restore dictionary.


// Load dictionary.
static	int		IAry_LoadDic( hIAry h, FILE *fp )
{
	int		ret;

	if ( ( ret = BAry_Load( &h->flag, fp ) ) != IARY_OK ||
		 ( ret = BAry_Load( &h->len, fp ) ) != IARY_OK ||
		 ( ret = BAry_Load( &h->val, fp ) ) != IARY_OK )
		return	ret;

	h->szint = h->nint = h->flag->nbit;
	h->szbit = h->nbit = h->val->nbit;

	return	IAry_BuildIndex( h );
}


// Interfaces.


// Create integer array.
int		IAry_Open( hIAry *h )
{
	int		ret;

	if ( ( *h = IAry_Create() ) == NULL )
		return	IARY_EMEM;

	if ( ( ret = IAry_Empty( *h ) ) != IARY_OK )
	{
		IAry_Destroy( *h );
		*h = NULL;
	}

	return	ret;
}

// Destroy integer array.
void	IAry_Close( hIAry h )
{
	IAry_Destroy( h );
}

// Load integer array.
int		IAry_Load( hIAry *h, const char *path )
{
	int		ret;
	FILE	*fp;

	if ( ( fp = fopen( path, "rb" ) ) == NULL )
		return	IARY_EIO;

	if ( ( ret = IAry_Open( h ) ) == IARY_OK &&
		 ( ret = IAry_LoadDic( *h, fp ) ) != IARY_OK )
	{
		IAry_Close( *h );
		*h = NULL;
	}

	fclose( fp );

	return	ret;
}

// Save integer array.[
int		IAry_Save( hIAry h, const char *path )
{
	int		ret;
	FILE	*fp;

	if ( ( fp = fopen( path, "wb" ) ) == NULL )
		return	IARY_EIO;

	( ret = BAry_Save( h->flag, fp ) ) == IARY_OK &&
	( ret = BAry_Save( h->len, fp ) ) == IARY_OK &&
	( ret = BAry_Save( h->val, fp ) ) == IARY_OK;

	fclose( fp );

	return	ret;
}

// Add integer.
int		IAry_AddInt( hIAry h, IAryInt x )
{
	int		ret;
	IAryLen		len;

	len = IAry_IntLen( x );
	if ( ( ret = IAry_Alloc( h, len ) ) != IARY_OK )
		return	ret;

	IAry_PutInt( h, x );

	return	IARY_OK;
}

// Add unsigned integer.
int		IAry_AddUInt( hIAry h, IAryUInt x )
{
	int		ret;
	IAryLen		len;

	len = IAry_UIntLen( x );
	if ( ( ret = IAry_Alloc( h, len ) ) != IARY_OK )
		return	ret;

	IAry_PutUInt( h, x );

	return	IARY_OK;
}

// Build index.
int		IAry_BuildIndex( hIAry h )
{
	int		ret;

	if ( ( ret = IAry_ResizeFlag( h, h->nint ) ) != IARY_OK ||
		 ( ret = IAry_ResizeOther( h, h->nbit ) ) != IARY_OK )
		return	ret;

	if ( ( ret = BAry_Resize( h->flag, h->nint ) ) != IARY_OK ||
		 ( ret = BAry_Resize( h->len, h->nbit + 1 ) ) != IARY_OK ||
		 ( ret = BAry_Resize( h->val, h->nbit ) ) != IARY_OK )
		return	ret;

	if ( ( ret = BAry_BuildRank( h->flag ) ) != IARY_OK ||
		 ( ret = BAry_BuildSelect( h->len ) ) != IARY_OK )
		return	ret;

	return	IARY_OK;
}

// Get integer.
IAryInt		IAry_GetInt( hIAry h, IAryIdx idx )
{
	IAryIdx		pos;

	if ( !BAry_GetBit( h->flag, idx ) )
		return	0;

	pos = IAry_GetPos( h, idx );

	return	IAry_IntPos( h, &pos );
}

// Get unsigned integer.
IAryUInt	IAry_GetUInt( hIAry h, IAryIdx idx )
{
	IAryIdx		pos;

	if ( !BAry_GetBit( h->flag, idx ) )
		return	0;

	pos = IAry_GetPos( h, idx );

	return	IAry_UIntPos( h, &pos );
}

// Start scan.
void	IAry_Start( hIAry h, hIAryScan scan )
{
	memset( scan, 0, sizeof( *scan ) );
	scan->h = h;
}

// Jump to another integer.
void	IAry_Jump( hIAryScan scan, IAryIdx idx )
{
	scan->idx = idx;
	scan->jump = 1;
}

// Get next integer.
IAryInt		IAry_NextInt( hIAryScan scan )
{
	if ( !BAry_GetBit( scan->h->flag, scan->idx ) )
	{
		scan->idx++;
		return	0;
	}

	IAry_JumpPos( scan );

	return	IAry_IntPos( scan->h, &scan->pos );
}

// Get next unsigned integer.
IAryUInt	IAry_NextUInt( hIAryScan scan )
{
	if ( !BAry_GetBit( scan->h->flag, scan->idx ) )
	{
		scan->idx++;
		return	0;
	}

	IAry_JumpPos( scan );

	return	IAry_UIntPos( scan->h, &scan->pos );
}

// Get size.
IAryIdx		IAry_Size( hIAry h )
{
	IAryIdx		size;

	size = sizeof( *h );
	size += BAry_Size( h->flag );
	size += BAry_Size( h->len );
	size += BAry_Size( h->val );

	return	size;
}
