#ifndef		__SUCCINCT_INTEGER_ARRAY_H_
#define		__SUCCINCT_INTEGER_ARRAY_H_


// Functions return the following codes.
#define		IARY_OK		0		// No error.
#define		IARY_EMEM	( -1 )	// Memory error.
#define		IARY_EIO	( -2 )	// I/O error.


#ifdef	__cplusplus
extern	"C"	{
#endif


// Integers.
typedef		unsigned long	IAryIdx;	// Index.
typedef		unsigned long	IAryLen;	// Length.
typedef		long			IAryInt;	// Integer.
typedef		unsigned long	IAryUInt;	// Unsigned integer.

// Compressed integer array.
typedef		struct IAry		*hIAry;

// Status of scan.
typedef		struct IAryScan
{
	hIAry		h;		// Target array.
	IAryIdx		idx;	// Index of next integer.
	IAryIdx		pos;	// Position of next integer.
	int			jump;	// Position is valid (0) or not (1).
}	IAryScan, *hIAryScan;


// Create integer array.
int		IAry_Open( hIAry *h );
// Destroy integer array.
void	IAry_Close( hIAry h );

// Load integer array.
int		IAry_Load( hIAry *h, const char *path );
// Save integer array.
int		IAry_Save( hIAry h, const char *path );

// Add integer.
int		IAry_AddInt( hIAry h, IAryInt x );
// Add unsigned integer.
int		IAry_AddUInt( hIAry h, IAryUInt x );

// Build index.
int		IAry_BuildIndex( hIAry h );

// Get integer.
IAryInt		IAry_GetInt( hIAry h, IAryIdx idx );
// Get unsigned integer.
IAryUInt	IAry_GetUInt( hIAry h, IAryIdx idx );

// Start scan.
void	IAry_Start( hIAry h, hIAryScan scan );
// Jump to another integer.
void	IAry_Jump( hIAryScan scan, IAryIdx idx );
// Get next integer.
IAryInt		IAry_NextInt( hIAryScan scan );
// Get next unsigned integer.
IAryUInt	IAry_NextUInt( hIAryScan scan );

// Get size.
IAryIdx		IAry_Size( hIAry h );


#ifdef	__cplusplus
}
#endif


#endif
