#include	<stdio.h>
#include	<stdlib.h>
#include	<time.h>

#include	"IntArray.h"


// Number of integers.
#define		N	80


// Population count (from Hacker's Delight).
static	unsigned long	PopCount( unsigned long x )
{
	x -= ( x >> 1 ) & 0x55555555;
	x = ( x & 0x33333333 ) + ( ( x >> 2 ) & 0x33333333 );
	x = ( x + ( x >> 4 ) ) & 0x0F0F0F0F;
	x += x >> 8;
	x += x >> 16;

	return	x & 0x3F;
}

// Add random integers.
void	TestAdd( hIAry h )
{
	IAryIdx		i, j;
	IAryInt		y;
	unsigned long	x;

	srand( time( NULL ) );

	printf( "Add:\n" );
	for ( i = 0; i < N; i += 20 )
	{
		for ( j = 0; j < 20 && i + j < N; j++ )
		{
			x = rand();
			if ( x >= RAND_MAX / 2 )
				y = 0;
			else
				y = PopCount( x ) - 8;

			printf( "%2ld ", y );
			IAry_AddInt( h, y );
		}
		printf( "\n" );
	}

	IAry_BuildIndex( h );
}

// Get integers.
void	TestGet( hIAry h )
{
	IAryIdx		i, j;
	IAryScan	scan;

	printf( "Get:\n" );
	for ( i = 0; i < N; i += 20 )
	{
		for ( j = 0; j < 20 && i + j < N; j++ )
			printf( "%2ld ", IAry_GetInt( h, i + j ) );
		printf( "\n" );
	}

	printf( "Scan:\n" );
	IAry_Start( h, &scan );
	for ( i = 0; i < N; i += 20 )
	{
		for ( j = 0; j < 20 && i + j < N; j++ )
			printf( "%2ld ", IAry_NextInt( &scan ) );
		printf( "\n" );
	}
}

int		main( int argc, char *argv[] )
{
	int		ret;
	hIAry	h;
	IAryIdx		size;

	// Initialize integer array.
	ret = IAry_Open( &h );
	fprintf( stderr, "IAry_Open(): %d\n", ret );

	// Add random integers.
	TestAdd( h );

	// Get integers.
	TestGet( h );

	// Get size.
	size = IAry_Size( h );
	printf( "Size: %lu bytes (%.2f bytes/integer)\n", size, 1.0 * size / N );

	// Destroy integer array.
	IAry_Close( h );

	return	0;
}
